﻿using UnityEngine;
using System.Collections;

[RequireComponent (typeof(Camera))]

public class HUD : MonoBehaviour {

	public SpriteRenderer leftMoveSpriteRenderer;
	public SpriteRenderer rightMoveSpriteRenderer;
	public SpriteRenderer jumpSpriteRenderer;
	public Transform movesLabel;
	public Transform restartLabel;

	private const float kFadeSpeed = 5.0f;

	private Transform leftMoveTransform;
	private Transform rightMoveTransform;
	private Transform jumpTransform;
	private Camera thisCamera;

	private readonly Vector3 offset = new Vector3(0.1f, 0.1f, 0.0f); // in % of camera orthographicSize
	private const float separator = 0.15f;

	void Start() {
		leftMoveTransform = leftMoveSpriteRenderer.transform;
		rightMoveTransform = rightMoveSpriteRenderer.transform;
		jumpTransform = jumpSpriteRenderer.transform;
		thisCamera = GetComponent<Camera>();
	}
	
	void Update() {

		float size = thisCamera.orthographicSize;
		float aspect = thisCamera.aspect;
		leftMoveTransform.localPosition = new Vector3(-size * aspect + size * offset.x , size - size * offset.y,  0.0f);
		jumpTransform.localPosition = new Vector3(-size * aspect + size * (offset.x + separator) , size - size * offset.y,  0.0f);
		rightMoveTransform.localPosition = new Vector3(-size * aspect + size * (offset.x + separator * 2) , size - size * offset.y,  0.0f);
		movesLabel.localPosition = new Vector3(-size * aspect + size * (offset.x + separator * 3) , size - size * offset.y,  0.0f);
		restartLabel.localPosition = new Vector3(size * aspect - size * offset.x, size - size * offset.y,  0.0f);

		AbilitiesManager abilitiesManager = AbilitiesManager.Instance;

		if (abilitiesManager.CanJump) {
			jumpSpriteRenderer.color = Color.Lerp(jumpSpriteRenderer.color, Color.white, Time.deltaTime * kFadeSpeed);
		}
		else {
			jumpSpriteRenderer.color = Color.Lerp(jumpSpriteRenderer.color, new Color(1.0f, 1.0f, 1.0f, 0.1f), Time.deltaTime * kFadeSpeed);
		}
	
		if (abilitiesManager.CanMoveLeft) {
			leftMoveSpriteRenderer.color = Color.Lerp(leftMoveSpriteRenderer.color, Color.white, Time.deltaTime * kFadeSpeed);
		}
		else {
			leftMoveSpriteRenderer.color = Color.Lerp(leftMoveSpriteRenderer.color, new Color(1.0f, 1.0f, 1.0f, 0.1f), Time.deltaTime * kFadeSpeed);
		}

		if (abilitiesManager.CanMoveRight) {
			rightMoveSpriteRenderer.color = Color.Lerp(rightMoveSpriteRenderer.color, Color.white, Time.deltaTime * kFadeSpeed);
		}
		else {
			rightMoveSpriteRenderer.color = Color.Lerp(rightMoveSpriteRenderer.color, new Color(1.0f, 1.0f, 1.0f, 0.1f), Time.deltaTime * kFadeSpeed);
		}
	}
}
