﻿using UnityEngine;
using System.Collections;

[RequireComponent (typeof(BoxCollider2D))]
[RequireComponent (typeof(Rigidbody2D))]

public class PlayerControllerSquare : MonoBehaviour {
		
	public float moveStrength = 1500.0f;
	public float rotationStrength = 6000.0f;
	public float spaceFriction = 10.0f;
	public float rotationFriction = 2.0f;
	public float brakeVelMul = 2.0f;
	public float maxHorizontalVelocity = 10.0f;
	public float maxAngularVelocity = 20.0f;
	public float jumpStrength = 2.0f;
	public AnimationCurve controlStrengthCurve;
	public FloorCheck floorCheck;
		
	private Rigidbody2D thisRigidbody2D;
	private float controlStrengthMul = 1.0f;
	private float timeSinceLastJump = 0.0f;

	private bool wasMovingLeft = false;
	private bool wasMovingRight = false;
	private bool didJump = false;

	void Start () {
	
		thisRigidbody2D = rigidbody2D;
	}
	
	void FixedUpdate () {

		Vector2 gravity = Physics2D.gravity;
		Vector2 rightVector = new Vector2(-gravity.y, gravity.x);
		rightVector.Normalize();
		Vector2 upVector = new Vector2(-rightVector.y, rightVector.x);
		float rightVelocityScalarProjection = Vector2.Dot(thisRigidbody2D.velocity, rightVector);
		float upVelocityScalarProjection = Vector2.Dot(thisRigidbody2D.velocity, upVector);
		Vector2 upVelocity =  upVelocityScalarProjection * upVector;
					
		// Movement
		if (Input.GetKey(KeyCode.LeftArrow) && AbilitiesManager.Instance.CanMoveLeft && Input.GetKey(KeyCode.RightArrow) == false) {

			if (rightVelocityScalarProjection > 0) {
				thisRigidbody2D.AddForce(new Vector3(-rightVector.x * moveStrength * controlStrengthMul * brakeVelMul * Time.fixedDeltaTime, -rightVector.y * moveStrength * controlStrengthMul * brakeVelMul * Time.fixedDeltaTime, 0.0f));
			}
			else if (rightVelocityScalarProjection <= 0) {
				thisRigidbody2D.AddForce(new Vector3(-rightVector.x * moveStrength * controlStrengthMul * Time.fixedDeltaTime, -rightVector.y * moveStrength * controlStrengthMul * Time.fixedDeltaTime, 0.0f));
			}

			// Don't rotate on the floor
			if (floorCheck.floorTransform == null) {
				if (thisRigidbody2D.angularVelocity < 0) {
					thisRigidbody2D.AddTorque(brakeVelMul * rotationStrength * Time.fixedDeltaTime);
				}
				else {
					thisRigidbody2D.AddTorque(rotationStrength * Time.fixedDeltaTime);
				}
			}

			wasMovingLeft = true;
		}

		if (Input.GetKey(KeyCode.RightArrow) && AbilitiesManager.Instance.CanMoveRight && Input.GetKey(KeyCode.LeftArrow) == false) {

			if (rightVelocityScalarProjection < 0) {
				thisRigidbody2D.AddForce(new Vector3(rightVector.x * moveStrength * controlStrengthMul * brakeVelMul * Time.fixedDeltaTime, rightVector.y * moveStrength * controlStrengthMul * brakeVelMul * Time.fixedDeltaTime, 0.0f));
			}
			else if (rightVelocityScalarProjection >= 0 ) { 
				thisRigidbody2D.AddForce(new Vector3(rightVector.x * moveStrength * controlStrengthMul * Time.fixedDeltaTime, rightVector.y * moveStrength * controlStrengthMul * Time.fixedDeltaTime, 0.0f));
			}

			// Don't rotate on the floor
			if (floorCheck.floorTransform == null) {
				if (thisRigidbody2D.angularVelocity > 0) {
					thisRigidbody2D.AddTorque(-brakeVelMul * rotationStrength * Time.fixedDeltaTime);
				}
				else {
					thisRigidbody2D.AddTorque(-rotationStrength * Time.fixedDeltaTime);
				}
			}

			wasMovingRight = true;
		}

		if (Input.GetKey(KeyCode.LeftArrow) == false && Input.GetKey(KeyCode.RightArrow) == false) {
			// Space friction
			thisRigidbody2D.AddForce(-rightVector * spaceFriction * controlStrengthMul * rightVelocityScalarProjection * Time.fixedDeltaTime);

			// Stop rotation in the air
			if (floorCheck.floorTransform == null) {
				thisRigidbody2D.AddTorque(-thisRigidbody2D.angularVelocity * 2.0f * Time.fixedDeltaTime);
			}
		}

		if (Input.GetKey(KeyCode.LeftArrow) == false) {
			if (wasMovingLeft) {
				AbilitiesManager.Instance.ForbidMoveLeft();
				wasMovingLeft = false;
			}
		}

		if (Input.GetKey(KeyCode.RightArrow) == false) {
			if (wasMovingRight) {
				AbilitiesManager.Instance.ForbidMoveRight();
				wasMovingRight = false;
			}
		}

		// Jump
		if (Input.GetKey(KeyCode.UpArrow) && floorCheck.floorTransform != null && timeSinceLastJump > 0.1f && AbilitiesManager.Instance.CanJump && didJump == false) {

			Vector2 rightVelocity = rightVelocityScalarProjection * rightVector;
			thisRigidbody2D.velocity = upVector * jumpStrength + rightVelocity;
			timeSinceLastJump = 0.0f;
			didJump = true;

			AbilitiesManager.Instance.ForbidJump();
		}
		// Limit horizontal velocity
		else if (rightVelocityScalarProjection > maxHorizontalVelocity) {			
			thisRigidbody2D.velocity = upVelocity + rightVector * maxHorizontalVelocity;
		}
		else if (rightVelocityScalarProjection < -maxHorizontalVelocity) {
			thisRigidbody2D.velocity = upVelocity - rightVector * maxHorizontalVelocity;
		}

		if (Input.GetKey(KeyCode.UpArrow) == false) {
			didJump = false;
		}

		// Limit angular velocity
		if (thisRigidbody2D.angularVelocity > maxAngularVelocity) {
			thisRigidbody2D.angularVelocity = maxAngularVelocity;
		}
		else if (thisRigidbody2D.angularVelocity < -maxAngularVelocity) {
			thisRigidbody2D.angularVelocity = -maxAngularVelocity;
		}

		timeSinceLastJump += Time.fixedDeltaTime;
	}

	void TemporaryDisableFriction() {

		controlStrengthMul = 0.0f;
		StopCoroutine("DisableFrictionCoroutine");
		StartCoroutine("DisableFrictionCoroutine", 0.5f);
	}

	IEnumerator DisableFrictionCoroutine(float time) {

		float elapsedTime = 0.0f;

		while (elapsedTime < time) {
			yield return 0;
			controlStrengthMul = controlStrengthCurve.Evaluate(elapsedTime / time);
			elapsedTime += Time.deltaTime;
		}
		controlStrengthMul = 1.0f;

	}
}
